@@ -336,6 +336,7 @@ grn_table_queue_init(grn_ctx *ctx, grn_table_queue *queue)
   queue->head = 0;
   queue->tail = 0;
   queue->cap = GRN_ARRAY_MAX;
+  queue->unblock_requested = GRN_FALSE;
   grn_table_queue_lock_init(ctx, queue);
 }
 
@@ -1130,8 +1131,9 @@ grn_array_pull(grn_ctx *ctx, grn_array *array, grn_bool blockp,
   grn_table_queue *queue = grn_array_queue(ctx, array);
   if (queue) {
     MUTEX_LOCK(queue->mutex);
+    queue->unblock_requested = GRN_FALSE;
     while (grn_table_queue_size(queue) == 0) {
-      if (!blockp) {
+      if (!blockp || queue->unblock_requested) {
         MUTEX_UNLOCK(queue->mutex);
         GRN_OUTPUT_BOOL(0);
         return id;
@@ -1150,6 +1152,18 @@ grn_array_pull(grn_ctx *ctx, grn_array *array, grn_bool blockp,
   return id;
 }
 
+void
+grn_array_unblock(grn_ctx *ctx, grn_array *array)
+{
+  grn_table_queue *queue = grn_array_queue(ctx, array);
+  if (!queue) {
+    return;
+  }
+
+  queue->unblock_requested = GRN_TRUE;
+  COND_BROADCAST(queue->cond);
+}
+
 /* grn_hash : hash table */
 
 #define GRN_HASH_MAX_SEGMENT  0x400
@@ -1318,22 +1332,22 @@ grn_hash_entry_get_key(grn_ctx *ctx, grn_hash *hash, grn_hash_entry *entry)
   if (hash->obj.header.flags & GRN_OBJ_KEY_VAR_SIZE) {
     if (grn_hash_is_io_hash(hash)) {
       if (entry->io_entry.flag & HASH_IMMEDIATE) {
-        return entry->io_entry.key.buf;
+        return (char *)entry->io_entry.key.buf;
       } else {
         return (char *)grn_io_hash_key_at(ctx, hash, entry->io_entry.key.offset);
       }
     } else {
       if (entry->tiny_entry.flag & HASH_IMMEDIATE) {
-        return entry->tiny_entry.key.buf;
+        return (char *)entry->tiny_entry.key.buf;
       } else {
         return entry->tiny_entry.key.ptr;
       }
     }
   } else {
     if (hash->key_size == sizeof(uint32_t)) {
-      return entry->plain_entry.key;
+      return (char *)entry->plain_entry.key;
     } else {
-      return entry->rich_entry.key_and_value;
+      return (char *)entry->rich_entry.key_and_value;
     }
   }
 }
@@ -3132,7 +3146,7 @@ subrecs_push(byte *subrecs, int size, int n_subrecs, int score, void *body, int
   while (n) {
     n2 = (n - 1) >> 1;
     c2 = GRN_RSET_SUBRECS_NTH(subrecs,size,n2);
-    if (GRN_RSET_SUBRECS_CMP(score, *c2, dir)) { break; }
+    if (GRN_RSET_SUBRECS_CMP(score, *c2, dir) > 0) { break; }
     GRN_RSET_SUBRECS_COPY(subrecs,size,n,c2);
     n = n2;
   }
@@ -3151,10 +3165,10 @@ subrecs_replace_min(byte *subrecs, int size, int n_subrecs, int score, void *bod
     n2 = n1 + 1;
     c1 = n1 < n_subrecs ? GRN_RSET_SUBRECS_NTH(subrecs,size,n1) : NULL;
     c2 = n2 < n_subrecs ? GRN_RSET_SUBRECS_NTH(subrecs,size,n2) : NULL;
-    if (c1 && GRN_RSET_SUBRECS_CMP(score, *c1, dir)) {
+    if (c1 && GRN_RSET_SUBRECS_CMP(score, *c1, dir) > 0) {
       if (c2 &&
-          GRN_RSET_SUBRECS_CMP(score, *c2, dir) &&
-          GRN_RSET_SUBRECS_CMP(*c1, *c2, dir)) {
+          GRN_RSET_SUBRECS_CMP(score, *c2, dir) > 0 &&
+          GRN_RSET_SUBRECS_CMP(*c1, *c2, dir) > 0) {
         GRN_RSET_SUBRECS_COPY(subrecs,size,n,c2);
         n = n2;
       } else {
@@ -3162,7 +3176,7 @@ subrecs_replace_min(byte *subrecs, int size, int n_subrecs, int score, void *bod
         n = n1;
       }
     } else {
-      if (c2 && GRN_RSET_SUBRECS_CMP(score, *c2, dir)) {
+      if (c2 && GRN_RSET_SUBRECS_CMP(score, *c2, dir) > 0) {
         GRN_RSET_SUBRECS_COPY(subrecs,size,n,c2);
         n = n2;
       } else {
@@ -3185,7 +3199,7 @@ grn_rhash_add_subrec(grn_hash *s, grn_rset_recinfo *ri, int score, void *body, i
     int ssize = s->subrec_size;
     int n_subrecs = GRN_RSET_N_SUBRECS(ri);
     if (limit < n_subrecs) {
-      if (GRN_RSET_SUBRECS_CMP(score, *ri->subrecs, dir)) {
+      if (GRN_RSET_SUBRECS_CMP(score, *ri->subrecs, dir) > 0) {
         subrecs_replace_min(ri->subrecs, ssize, limit, score, body, dir);
       }
     } else {
